import matplotlib.pyplot as plt
import numpy as np
import stan
import arviz as az
"""
         データは、Spiegelhalter(2019)のウェブサイト
         https://github.com/dspiegel29/ArtofStatistics
         からダウンロードしたファイル09-4-homocide-trends-x.csv
         に含まれているものである。
"""
years = np.array([1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
                  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
                  2013, 2014, 2015])
counts = np.array([587, 609, 648, 676, 761, 736, 729, 749, 639,
                   676, 712, 617, 574, 599, 497, 508, 506, 497, 557])

for v in zip(years, counts):
    print(v)

years_c = years - np.mean(years)
years_c = np.array(years_c)
years = np.array(years)
counts = np.array(counts, dtype=int)

Loos = np.empty(6)
waics = np.empty(6)

plt.figure(figsize=(12,5))

for p in range(1, 7):
    #
    #            MCMCサンプリング
    #
    def init_f():
        return {'v_a': np.mean(counts), 'v_b':[0.0]*p}

    with open('poisson_reg_dp_loo_waic.stan') as f_stan:
        sm = stan.build(f_stan.read(),
                        dict({'p':p, 'n':len(years_c), 'x':years_c, 'y':counts}))

    fit = sm.sample(num_chains=4, init=[init_f()]*4)

    i_data = az.from_pystan(posterior=fit, log_likelihood='LL', posterior_model=sm)

    print(az.summary(i_data))

    Loo = az.loo(i_data, scale='deviance')
    Loos[p-1] = Loo.elpd_loo
    waic = az.waic(i_data, scale='deviance')
    waics[p-1] = waic.elpd_waic

    plt.subplot(int(f'23{p}'))
    
    plt.bar(years, counts, label=f'p={p}')
    
    d_fm = fit.to_frame()
    med_a = np.median(d_fm['a'])
    med_b = np.empty(p)
    for j in range(p):
        med_b[j] = np.median(d_fm[f'b.{j+1}'])

    def reg_p(x):
        v = med_a;
        for i in range(p):
            v += med_b[i] * x**(i+1)
        return v

    reg_v = reg_p(years_c)
    plt.plot(years, reg_v)
    plt.legend()

plt.tight_layout()
plt.show()

for p in range(1,7):
    print('p =', p)
    print('loo =', Loos[p-1])
    print('waic =', waics[p-1])

