#import matplotlib.pyplot as plt
import numpy as np
import stan
import arviz as az
import scipy.stats as ss
import pickle
"""
         データは、Spiegelhalter(2019)のウェブサイト
         https://github.com/dspiegel29/ArtofStatistics
         からダウンロードしたファイル09-4-homocide-trends-x.csv
         に含まれているものである。
"""
years = np.array([1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
                  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
                  2013, 2014, 2015])
counts = np.array([587, 609, 648, 676, 761, 736, 729, 749, 639,
                   676, 712, 617, 574, 599, 497, 508, 506, 497, 557])
years_c = years - np.mean(years)       #     中心化

p=4                                    #    4次の多項式を用いる
#
#                 MCMCサンプリング
#
def init_f():
    return {'v_a': np.mean(counts), 'v_b':[0.0]*p}   #   MCMCの初期値

with open('poisson_reg_dp_loo_waic.stan') as f_stan:
    sm = stan.build(f_stan.read(),
                    dict({'p':p, 'n':len(years_c), 'x':years_c, 'y':counts}))

fit = sm.sample(num_chains=4, init=[init_f()]*4)

i_data = az.from_pystan(posterior=fit, log_likelihood='LL', posterior_model=sm)
print(az.summary(i_data))

with open('i_data.pkl', 'wb') as f:
    pickle.dump(i_data, f)

#plt.figure(figsize=(9,7))
#plt.bar(years, counts) 

d_fm = fit.to_frame()

with open('d_fm.pkl', 'wb') as f:
    pickle.dump(d_fm, f)

