import matplotlib.pyplot as plt
import numpy as np
#import stan
import arviz as az
import scipy.stats as ss
import pickle
"""
         データは、Spiegelhalter(2019)のウェブサイト
         https://github.com/dspiegel29/ArtofStatistics
         からダウンロードしたファイル09-4-homocide-trends-x.csv
         に含まれているものである。
"""
years = np.array([1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
                  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
                  2013, 2014, 2015])
counts = np.array([587, 609, 648, 676, 761, 736, 729, 749, 639,
                   676, 712, 617, 574, 599, 497, 508, 506, 497, 557])
years_c = years - np.mean(years)       #     中心化

p=4                                    #    4次の多項式を用いる

plt.figure(figsize=(9,7))
plt.bar(years, counts) 

with open('i_data.pkl', 'rb') as f:
    i_data = pickle.load(f)
with open('d_fm.pkl', 'rb') as f:
    d_fm = pickle.load(f)

#
#         4次の多項式
#
def reg_4(x, va, b1, b2, b3, b4):
    return va + b1*x + b2*(x**2) + b3*(x**3) + b4*(x**4)

smpl_a = d_fm['a']
smpl_b1 = d_fm['b.1']
smpl_b2 = d_fm['b.2']
smpl_b3 = d_fm['b.3']
smpl_b4 = d_fm['b.4']
#
#       MCMCサンプルからランダムに1000組のサンプルを選び
#       それぞれのサンプル値による回帰曲線を描く
#
n_samples = len(smpl_a)
rng = np.random.default_rng()
boot_idx = rng.choice(n_samples, 1000)     #    ランダムに1000個採るための添字
regs = []
for i in range(1000):
    idx = boot_idx[i]
    reg_v = reg_4(years_c, smpl_a[idx], smpl_b1[idx], smpl_b2[idx],
                  smpl_b3[idx], smpl_b4[idx])
    regs.append(reg_v)
    if (i % 3) == 0:        #    ランダムな回帰曲線を間引いて描画
        plt.plot(years, reg_v, c='orange', ls=':', alpha=0.1)
#
#     ランダムに選んだ回帰曲線の2.5%値と97.5%値を描く
#
reg025_975 = np.percentile(regs, [2.5, 97.5], axis=0)
plt.plot(years, reg025_975[0], c='k', ls='--', label='2.5% of regs')
plt.plot(years, reg025_975[1], c='k', ls='-.', label='97.5% of regs')
#
#　　　　パラメータの事後分布の中央値を係数値とする
#        回帰曲線を描く
#
reg_v = reg_4(years_c, np.median(smpl_a), np.median(smpl_b1),
              np.median(smpl_b2), np.median(smpl_b3),
              np.median(smpl_b4))
plt.plot(years, reg_v, c='k', label='Reg.by median\ndegree = 4')
#
#         中央値を係数値とする回帰曲線に対応するポアッソン分布の
#         2.5%値と97.5%値を端点する区間を描く
#
pp025 = []
pp975 = []
for v in reg_v:
    pp025.append(ss.poisson.ppf(0.025, v))
    pp975.append(ss.poisson.ppf(0.975, v))    
for i, v in enumerate(years):
    plt.plot([v,v], [pp025[i], pp975[i]], lw=5,
             markersize=20, marker='+', c='lime')

plt.plot([], lw=5, c='lime', label='95% interval')
plt.title('Homicides Each Year in England and Wales', fontsize=16)
plt.xticks(years, years, rotation='vertical')
plt.xlabel('Year', fontsize=18)
plt.ylabel('Count of Incidents', fontsize=14)
plt.legend()
plt.tight_layout()
plt.show()
