//
//          ポアッソン回帰モデル
//           Y. Okamoto, 2024.08
//
data {
    int p;            //  多項式の次数
    int n;            //  データ数
    array[n] real x;  //  独立変数
    array[n] int y;   //  従属変数
}
transformed data {
    real mean_y;
    mean_y = mean(y);

    array[n] vector[p] Xs;     //    独立変数のj乗の値の配列
    for (i in 1:n)
        for (j in 1:p) {
                Xs[i][j] = x[i] ^ j;
        }
}
parameters {
    //
    //     PyStan 3の要請に合わせたパラメータ
    //     transformed parametersでモデルに合うパラメータに変換
    //
    real<lower=0.0> v_a;
    vector[p] v_b;
}
transformed parameters {
    real a;                //   回帰式のｙ切片
    vector[p] b;           //   回帰式の偏回帰係数
    array[n] real lmbd;    //   ポアッソン分布のパラメタ
    //
    //       リスケーリング
    //
    a = mean_y * v_a;
    for (j in 1:p)
        b[j] = v_b[j] * mean_y/(10^j);
    //
    //     ポアッソン分布のパラメータlmbdの回帰式
    //
    for (i in 1:n) {
        lmbd[i] = a;
        for (j in 1:p) {
            lmbd[i] += b[j] * Xs[i][j];
            if (lmbd[i] <= 0.0)    //   非正値は強制的に変更
                lmbd[i] = 0.001;
        }
    }
}
model {
    v_a ~ exponential(0.1);
    v_b ~ normal(0.0, 10.0);
    for (i in 1:n)
        y[i] ~ poisson(lmbd[i]);       //    ポアッソン回帰モデル
}
generated quantities {
    array[n] real LL;       //    対数尤度
    for (i in 1:n)
        LL[i] = poisson_lpmf(y[i] | lmbd[i]);
}
